"use strict";
/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   THIRD PARTY LICENSE NOTICE:

   AsyncQueue is derived from the implementation of Queue in
   Promise Extensions for Javascript: https://github.com/rbuckton/prex

   Promise Extensions is licensed under the Apache 2.0 License:

   Promise Extensions for JavaScript
   Copyright (c) Microsoft Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WaitQueue = void 0;
const cancelable_1 = require("@esfx/cancelable");
const internal_linked_list_1 = function () {
    var module = { exports: {} };
    (function (module, exports, require) {
        "use strict";
        Object.defineProperty(exports, "__esModule", { value: true });
        function listCreate() {
            return { size: 0, head: null };
        }
        exports.listCreate = listCreate;
        function listAdd(list, value) {
            const node = { value, next: null, prev: null };
            if (list.head === null) {
                list.head = node.next = node.prev = node;
            }
            else {
                const tail = list.head.prev;
                if (!(tail === null || tail === void 0 ? void 0 : tail.next))
                    throw new Error("Illegal state");
                node.prev = tail;
                node.next = tail.next;
                tail.next = tail.next.prev = node;
            }
            list.size++;
            return node;
        }
        exports.listAdd = listAdd;
        function listRemove(list, node) {
            if (node.next === null || node.prev === null)
                return false;
            if (node.next === node) {
                list.head = null;
            }
            else {
                node.next.prev = node.prev;
                node.prev.next = node.next;
                if (list.head === node) {
                    list.head = node.next;
                }
            }
            node.next = node.prev = null;
            list.size--;
            return true;
        }
        exports.listRemove = listRemove;
    })(module, module.exports, null);
    return module.exports;
}();
/**
 * An async coordination primitive that provides a queue of Promises.
 */
class WaitQueue {
    constructor() {
        this._list = (0, internal_linked_list_1.listCreate)();
    }
    /**
     * Gets the number of pending entries in the queue.
     */
    get size() {
        return this._list.size;
    }
    /**
     * Returns a `Promise` for the next value to be added to the queue.
     * @param cancelable A `Cancelable` used to cancel the request.
     */
    wait(cancelable) {
        return new Promise((resolve, reject) => {
            var _b;
            if (!(cancelable === undefined) && !cancelable_1.Cancelable.hasInstance(cancelable))
                throw new TypeError("Cancelable expected: cancelable");
            const signal = cancelable === null || cancelable === void 0 ? void 0 : cancelable[cancelable_1.Cancelable.cancelSignal]();
            if (signal === null || signal === void 0 ? void 0 : signal.signaled)
                throw (_b = signal.reason) !== null && _b !== void 0 ? _b : new cancelable_1.CancelError();
            const node = (0, internal_linked_list_1.listAdd)(this._list, {
                unsubscribe() {
                    subscription === null || subscription === void 0 ? void 0 : subscription.unsubscribe();
                },
                resolve,
                reject,
            });
            const subscription = signal === null || signal === void 0 ? void 0 : signal.subscribe(() => {
                var _b;
                const entry = node.value;
                if ((0, internal_linked_list_1.listRemove)(this._list, node)) {
                    entry.reject((_b = signal.reason) !== null && _b !== void 0 ? _b : new cancelable_1.CancelError());
                }
            });
        });
    }
    /**
     * Resolves a pending `wait()` operation with the provided value.
     * @returns `true` if a pending `wait()` operation was resolved; otherwise, `false`.
     */
    resolveOne(value) {
        const node = this._list.head;
        if (node) {
            const pending = node.value;
            if ((0, internal_linked_list_1.listRemove)(this._list, node)) {
                pending.unsubscribe();
                pending.resolve(value);
                return true;
            }
        }
        return false;
    }
    /**
     * Resolves all pending `wait()` operations with the provided value.
     * @returns The number of pending `wait()` operations that were resolved.
     */
    resolveAll(value) {
        let count = 0;
        while (this.resolveOne(value)) {
            count++;
        }
        return count;
    }
    /**
     * Rejects the next pending `wait()` operation with the provided reason.
     * @returns `true` if a pending `wait()` operation was rejected; otherwise, `false`.
     */
    rejectOne(reason) {
        const node = this._list.head;
        if (node) {
            const pending = node.value;
            if ((0, internal_linked_list_1.listRemove)(this._list, node)) {
                pending.unsubscribe();
                pending.reject(reason);
                return true;
            }
        }
        return false;
    }
    /**
     * Rejects all pending `wait()` operations with the provided reason.
     * @returns The number of pending `wait()` operations that were rejected.
     */
    rejectAll(reason) {
        let count = 0;
        while (this.rejectOne(reason)) {
            count++;
        }
        return count;
    }
    /**
     * Rejects the next pending `wait()` operation with a `CancelError`.
     * @returns `true` if a pending `wait()` operation was rejected; otherwise, `false`.
     */
    cancelOne(reason = new cancelable_1.CancelError()) {
        return this.rejectOne(reason);
    }
    /**
     * Rejects all pending `wait()` operations with a `CancelError`.
     * @returns The number of pending `wait()` operations that were rejected.
     */
    cancelAll(reason = new cancelable_1.CancelError()) {
        return this.rejectAll(reason);
    }
}
exports.WaitQueue = WaitQueue;
_a = WaitQueue;
(() => {
    Object.defineProperty(_a.prototype, Symbol.toStringTag, { configurable: true, value: "WaitQueue" });
})();
//# sourceMappingURL=index.js.map